from django.contrib.auth.mixins import PermissionRequiredMixin
from django.db.models import Q
from django.shortcuts import get_object_or_404
from django.urls import reverse, reverse_lazy
from django.utils.http import urlencode
from django.views.generic import CreateView, ListView, DetailView, UpdateView, DeleteView

from webapp.forms import PostForm, SearchForm
from webapp.models import Post, Comment


class PostListView(ListView):
    model = Post
    template_name = 'index.html'
    context_object_name = 'posts'
    ordering = ['-create']
    paginate_by = 10

    def get_queryset(self):
        if self.request.user.is_superuser:
            queryset = super().get_queryset().filter(is_active=True)
            return queryset
        return super().get_queryset()


class PostDetailView(DetailView):
    model = Post
    template_name = 'post/post_detail.html'
    context_object_name = 'post'
    pk_url_kwarg = 'pk'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        comments = self.get_object().comments.order_by('-create')
        context['comments'] = comments
        return context


class PostCreateView(PermissionRequiredMixin, CreateView):
    model = Post
    template_name = 'post/post_create.html'
    form_class = PostForm
    permission_required = 'webapp.add_post'

    def get_success_url(self):
        return reverse('webapp:post_detail', kwargs={"pk": self.object.pk})


class PostUpdateView(PermissionRequiredMixin, UpdateView):
    model = Post
    context_object_name = 'post'
    template_name = 'post/post_update.html'
    form_class = PostForm
    permission_required = 'webapp.change_post'

    def get_success_url(self):
        return reverse('webapp:post_detail', kwargs={"pk": self.object.pk})


class PostDeleteView(PermissionRequiredMixin, DeleteView):
    model = Post
    template_name = 'post/post_delete.html'
    pk_url_kwarg = 'pk'
    success_url = reverse_lazy('webapp:index')
    permission_required = 'webapp.delete_post'


class CommentCreateView(CreateView):
    model = Comment
    template_name = 'comment/comment_form.html'
    fields = ['text']

    def form_valid(self, form):
        form.instance.post = self.get_object()
        form.instance.author = self.request.user
        return super().form_valid(form)

    def get_object(self, queryset=None):
        pk = self.kwargs.get("p_pk")
        return get_object_or_404(Post, pk=pk)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['post'] = self.get_object()
        return self.get_context_data(context)

    def get_success_url(self):
        return reverse('webapp:post_detail', kwargs={"pk": self.get_object().pk})


class CommentDeleteView(PermissionRequiredMixin, DeleteView):
    model = Comment
    template_name = 'comment/comment_delete.html'
    pk_url_kwarg = 'pk'
    permission_required = 'webapp.delete_comment'

    def has_permission(self):
        self.object = self.get_object()
        return super().has_permission() or self.object.author == self.request.user

    def get_success_url(self):
        return reverse('webapp:post_detail', kwargs={"pk": self.object.post.pk})


class SearchView(ListView):
    model = Post
    template_name = 'post/search_list.html'
    context_object_name = 'posts'

    def get(self, request, *args, **kwargs):
        self.form = self.get_search_form()
        self.search_value = self.get_search_value()
        return super().get(request, *args, **kwargs)

    def get_context_data(self, *, object_list=None, **kwargs):
        context = super().get_context_data(object_list=object_list, **kwargs)
        context['search_form'] = self.form
        if self.search_value:
            context['query'] = urlencode({'q': self.search_value})
        return context

    def get_queryset(self):
        query_set = super().get_queryset().filter(is_active=True)
        if self.search_value:
            query = Q(title__icontains=self.search_value) | Q(text__icontains=self.search_value)
            query_set = query_set.filter(query)
            return query_set
        else:
            return query_set


    def get_search_form(self):
        return SearchForm(self.request.GET)

    def get_search_value(self):
        if self.form.is_valid():
            return self.form.cleaned_data['q']

